package org.xbib.interlibrary.z;

import java.util.Collection;
import java.util.List;
import java.util.Map;
import java.util.function.Supplier;
import org.xbib.datastructures.tiny.TinyMap;
import org.xbib.interlibrary.api.util.Lists;
import org.xbib.marc.MarcRecord;

public class DisplayHelper {

    private DisplayHelper() {
    }

    public static void display(TinyMap.Builder<String, Object> builder,
                               MarcHelper marcHelper,
                               MarcRecord marcRecord,
                               String name,
                               String label,
                               String url,
                               Supplier<String> idSupplier) {
        putIf(builder, "servicename", name);
        putIf(builder, "servicelabel", label);
        putIf(builder, "serviceurl", url);
        putIf(builder, "id", idSupplier.get());
        putIf(builder, "identifiers", marcHelper.extractIdentifiers(marcRecord, "035"));
        putIf(builder, "date", marcHelper.extractDate(marcRecord));
        putIf(builder, "creator", marcHelper.extractCreators(marcRecord, List.of("100", "700")));
        putIf(builder, "contributor", marcHelper.extractCreators(marcRecord, List.of("200")));
        putIf(builder, "title", marcHelper.extractTitle(marcRecord, List.of("245", "490")));
        putIf(builder, "country", marcHelper.extractCountry(marcRecord, "044", "c"));
        putIf(builder, "language", marcHelper.extractLanguage(marcRecord, "041", "a"));
        putIf(builder, "publication", marcHelper.extractPublication(marcRecord, List.of("250", "515"), "a"));
        putIf(builder, "publishedAt", marcHelper.extractPublishedAt(marcRecord, List.of("260", "264"), "a"));
        putIf(builder, "publishedBy", marcHelper.extractPublishedBy(marcRecord, List.of("260", "264"), "b"));
        putIf(builder, "publishedOn", marcHelper.extractPublishedOn(marcRecord, List.of("260", "264"), "c"));
        putIf(builder, "isbn", marcHelper.extractIBN(marcRecord, "020", null, List.of("a")));
        putIf(builder, "issn", marcHelper.extractIBN(marcRecord, "022", null, List.of("a")));
        putIf(builder, "description", Lists.join(marcHelper.extractExtent(marcRecord, "300", "a"),
                marcHelper.extractExtent(marcRecord, "300", "b")));
        List<String> type = marcHelper.extractTypeCodes(marcRecord);
        putIf(builder, "type", type);
        putIf(builder, "format", marcHelper.extractFormatCodes(marcRecord));
        putIf(builder, "link", marcHelper.extractLinks(marcRecord));
        putIf(builder, "in", marcHelper.extractEmbedments(marcRecord));
        putIf(builder, "chronology", marcHelper.extractChronology(marcRecord, "362", "a"));
        putIf(builder, "material", marcHelper.extractMaterialAsString(marcRecord));
        putIf(builder, "multipart", marcHelper.extractMultipart(marcRecord));
    }

    private static void putIf(Map<String, Object> map, String key, String value) {
        if (key != null && value != null && !"null".equals(value)) {
            map.put(key, value);
        }
    }

    private static void putIf(Map<String, Object> map, String key, Collection<?> value) {
        if (key != null && value != null && !value.isEmpty()) {
            map.put(key, value);
        }
    }

    private static void putIf(Map<String, Object> map, String key, Map<?,?> value) {
        if (key != null && value != null && !value.isEmpty()) {
            map.put(key, value);
        }
    }

    private static void putIf(Map<String, Object> map, String key, boolean value) {
        if (key != null) {
            map.put(key, value);
        }
    }
}
